@@ -10,13 +10,10 @@
 """Test suite for pptx.presentation module."""
 
 import gc
-import inspect
 import os
 import re
 
-from hamcrest import (assert_that, has_item, is_, is_in, is_not, equal_to,
-                      greater_than)
-from lxml import etree
+from hamcrest import assert_that, is_, is_in, is_not, equal_to
 from StringIO import StringIO
 
 try:
@@ -28,25 +25,29 @@ from mock import Mock, patch, PropertyMock
 
 import pptx.presentation
 
-from pptx.packaging import prettify_nsdecls
-
-from pptx.presentation import (Package, Collection, _RelationshipCollection,
-    _Relationship, Presentation, PartCollection, BasePart, Part,
-    SlideCollection, BaseSlide, Slide, SlideLayout, SlideMaster, Image,
-    ShapeCollection, BaseShape, Shape, Placeholder, TextFrame, _Font,
-    Paragraph, Run, _to_unicode)
-
 from pptx.constants import MSO
 from pptx.exceptions import InvalidPackageError
+
+from pptx.oxml import _SubElement, oxml_fromstring, oxml_tostring, oxml_parse
+
+from pptx.packaging import prettify_nsdecls
+
+from pptx.presentation import (
+    Package, Collection, _RelationshipCollection, _Relationship, Presentation,
+    PartCollection, BasePart, Part, SlideCollection, BaseSlide, Slide,
+    SlideLayout, SlideMaster, Image, ShapeCollection, BaseShape, Shape,
+    Placeholder, TextFrame, _Font, Paragraph, Run, _to_unicode)
+
 from pptx.spec import namespaces, qtag
-from pptx.spec import (CT_PRESENTATION, CT_SLIDE, CT_SLIDELAYOUT,
-    CT_SLIDEMASTER)
-from pptx.spec import (RT_HANDOUTMASTER, RT_IMAGE, RT_NOTESMASTER,
-    RT_OFFICEDOCUMENT, RT_PRESPROPS, RT_SLIDE, RT_SLIDELAYOUT, RT_SLIDEMASTER,
-    RT_TABLESTYLES, RT_THEME, RT_VIEWPROPS)
-from pptx.spec import (PH_TYPE_CTRTITLE, PH_TYPE_DT, PH_TYPE_FTR, PH_TYPE_OBJ,
-    PH_TYPE_SLDNUM, PH_TYPE_SUBTITLE, PH_TYPE_TBL, PH_TYPE_TITLE,
-    PH_ORIENT_HORZ, PH_ORIENT_VERT, PH_SZ_FULL, PH_SZ_HALF, PH_SZ_QUARTER)
+from pptx.spec import (
+    CT_PRESENTATION, CT_SLIDE, CT_SLIDELAYOUT, CT_SLIDEMASTER)
+from pptx.spec import (
+    RT_IMAGE, RT_OFFICEDOCUMENT, RT_PRESPROPS, RT_SLIDE, RT_SLIDELAYOUT,
+    RT_SLIDEMASTER)
+from pptx.spec import (
+    PH_TYPE_CTRTITLE, PH_TYPE_DT, PH_TYPE_FTR, PH_TYPE_OBJ, PH_TYPE_SLDNUM,
+    PH_TYPE_SUBTITLE, PH_TYPE_TBL, PH_TYPE_TITLE, PH_ORIENT_HORZ,
+    PH_ORIENT_VERT, PH_SZ_FULL, PH_SZ_HALF, PH_SZ_QUARTER)
 from pptx.util import Inches, Px, Pt
 from testing import TestCase
 
@@ -69,55 +70,58 @@ def absjoin(*paths):
 thisdir = os.path.split(__file__)[0]
 test_file_dir = absjoin(thisdir, 'test_files')
 
-test_image_path  = absjoin(test_file_dir, 'python-icon.jpeg')
-test_bmp_path    = absjoin(test_file_dir, 'python.bmp')
-new_image_path   = absjoin(test_file_dir, 'monty-truth.png')
-test_pptx_path   = absjoin(test_file_dir, 'test.pptx')
+test_image_path = absjoin(test_file_dir, 'python-icon.jpeg')
+test_bmp_path = absjoin(test_file_dir, 'python.bmp')
+new_image_path = absjoin(test_file_dir, 'monty-truth.png')
+test_pptx_path = absjoin(test_file_dir, 'test.pptx')
 images_pptx_path = absjoin(test_file_dir, 'with_images.pptx')
 
 nsmap = namespaces('a', 'r', 'p')
-nsprefix_decls = (' xmlns:p="http://schemas.openxmlformats.org/presentationml'
-    '/2006/main" xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/ma'
-    'in" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relat'
-    'ionships"')
+nsprefix_decls = (
+    ' xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main" xm'
+    'lns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="h'
+    'ttp://schemas.openxmlformats.org/officeDocument/2006/relationships"')
+
 
 def _empty_spTree():
     xml = ('<p:spTree xmlns:p="http://schemas.openxmlformats.org/'
            'presentationml/2006/main" xmlns:a="http://schemas.openxmlformats.'
            'org/drawingml/2006/main"><p:nvGrpSpPr><p:cNvPr id="1" name=""/>'
            '<p:cNvGrpSpPr/><p:nvPr/></p:nvGrpSpPr><p:grpSpPr/></p:spTree>')
-    return etree.fromstring(xml)
+    return oxml_fromstring(xml)
+
 
 def _sldLayout1():
     path = os.path.join(thisdir, 'test_files/slideLayout1.xml')
-    sldLayout = etree.parse(path).getroot()
+    sldLayout = oxml_parse(path).getroot()
     return sldLayout
 
+
 def _sldLayout1_shapes():
     sldLayout = _sldLayout1()
     spTree = sldLayout.xpath('./p:cSld/p:spTree', namespaces=nsmap)[0]
     shapes = ShapeCollection(spTree)
     return shapes
 
+
 def _strip_nsdecls(xml):
     ptrn_str = ' xmlns(:[a-z]+)="[a-zA-Z0-9:/.]+"'
     nsdecl_re = re.compile(ptrn_str)
     return nsdecl_re.sub('', xml)
 
+
 def _txbox_xml():
-    xml = (\
-        "<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n<p:sp xmlns"
-        ':r="http://schemas.openxmlformats.org/officeDocument/2006/relationsh'
-        'ips"\n      xmlns:p="http://schemas.openxmlformats.org/presentationm'
-        'l/2006/main"\n      xmlns:a="http://schemas.openxmlformats.org/drawi'
-        'ngml/2006/main">\n  <p:nvSpPr>\n    <p:cNvPr id="2" name="TextBox 1"'
-        '/>\n    <p:cNvSpPr txBox="1"/>\n    <p:nvPr/>\n  </p:nvSpPr>\n  <p:s'
-        'pPr>\n    <a:xfrm>\n      <a:off x="914400" y="1828800"/>\n      <a:'
-        'ext cx="1371600" cy="457200"/>\n    </a:xfrm>\n    <a:prstGeom prst='
-        '"rect">\n      <a:avLst/>\n    </a:prstGeom>\n    <a:noFill/>\n  </p'
-        ':spPr>\n  <p:txBody>\n    <a:bodyPr wrap="none">\n      <a:spAutoFit'
-        '/>\n    </a:bodyPr>\n    <a:lstStyle/>\n    <a:p/>\n  </p:txBody>\n<'
-        '/p:sp>' )
+    xml = (
+        '<?xml version=\'1.0\' encoding=\'UTF-8\' standalone=\'yes\'?>\n<p:sp'
+        ' xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main'
+        '"\n      xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/m'
+        'ain">\n  <p:nvSpPr>\n    <p:cNvPr id="2" name="TextBox 1"/>\n    <p:'
+        'cNvSpPr txBox="1"/>\n    <p:nvPr/>\n  </p:nvSpPr>\n  <p:spPr>\n    <'
+        'a:xfrm>\n      <a:off x="914400" y="1828800"/>\n      <a:ext cx="137'
+        '1600" cy="457200"/>\n    </a:xfrm>\n    <a:prstGeom prst="rect">\n  '
+        '    <a:avLst/>\n    </a:prstGeom>\n    <a:noFill/>\n  </p:spPr>\n  <'
+        'p:txBody>\n    <a:bodyPr wrap="none">\n      <a:spAutoFit/>\n    </a'
+        ':bodyPr>\n    <a:lstStyle/>\n    <a:p/>\n  </p:txBody>\n</p:sp>')
     return xml
 
 
@@ -138,10 +142,8 @@ class PartBuilder(object):
 
 class RelationshipCollectionBuilder(object):
     """Builder class for test RelationshipCollections"""
-    partname_tmpls =\
-        { RT_SLIDEMASTER : '/ppt/slideMasters/slideMaster%d.xml'
-        , RT_SLIDE       : '/ppt/slides/slide%d.xml'
-        }
+    partname_tmpls = {RT_SLIDEMASTER: '/ppt/slideMasters/slideMaster%d.xml',
+                      RT_SLIDE:       '/ppt/slides/slide%d.xml'}
 
     def __init__(self):
         self.relationships = []
@@ -233,7 +235,7 @@ class TestBasePart(TestCase):
     def test__blob_value_for_xml_part(self):
         """BasePart._blob value is correct for XML part"""
         # setup -----------------------
-        elm = etree.fromstring('<root><elm1 attr="one"/></root>')
+        elm = oxml_fromstring('<root><elm1 attr="one"/></root>')
         self.basepart._element = elm
         self.basepart.partname = '/ppt/presentation.xml'
         # exercise --------------------
@@ -263,7 +265,7 @@ class TestBasePart(TestCase):
         elm = part._element
         # verify ----------------------
         expected = '<root><elm1 attr="spam"/></root>'
-        actual = etree.tostring(elm)
+        actual = oxml_tostring(elm)
         msg = "expected '%s', got '%s'" % (expected, actual)
         self.assertEqual(expected, actual, msg)
 
@@ -298,15 +300,11 @@ class TestBaseShape(TestCase):
     """Test BaseShape"""
     def setUp(self):
         path = os.path.join(thisdir, 'test_files/slide1.xml')
-        self.sld = etree.parse(path).getroot()
+        self.sld = oxml_parse(path).getroot()
         xpath = './p:cSld/p:spTree/p:pic'
         pic = self.sld.xpath(xpath, namespaces=nsmap)[0]
         self.base_shape = BaseShape(pic)
 
-    def test_class_present(self):
-        """BaseShape class present in presentation module"""
-        self.assertClassInModule(pptx.presentation, 'BaseShape')
-
     def test_has_textframe_value(self):
         """BaseShape.has_textframe value correct"""
         # setup -----------------------
@@ -451,10 +449,6 @@ class TestCollection(TestCase):
     def setUp(self):
         self.collection = Collection()
 
-    def test_class_present(self):
-        """Collection class present in presentation module"""
-        self.assertClassInModule(pptx.presentation, 'Collection')
-
     def test_indexable(self):
         """Collection is indexable (e.g. no TypeError on 'collection[0]')"""
         # verify ----------------------
@@ -504,14 +498,14 @@ class Test_Font(TestCase):
     """Test _Font class"""
     def setUp(self):
         self.rPr_xml = '<a:rPr%s/>' % nsprefix_decls
-        self.rPr = etree.fromstring(self.rPr_xml)
+        self.rPr = oxml_fromstring(self.rPr_xml)
         self.font = _Font(self.rPr)
 
     def test_get_bold_setting(self):
         """_Font.bold returns True on bold font weight"""
         # setup -----------------------
         rPr_xml = '<a:rPr%s b="1"/>' % nsprefix_decls
-        rPr = etree.fromstring(rPr_xml)
+        rPr = oxml_fromstring(rPr_xml)
         font = _Font(rPr)
         # verify ----------------------
         assert_that(self.font.bold, is_(False))
@@ -520,36 +514,44 @@ class Test_Font(TestCase):
     def test_set_bold(self):
         """Setting _Font.bold to True selects bold font weight"""
         # setup -----------------------
-        expected_rPr_xml = '<a:rPr%s b="1"/>'% (nsprefix_decls)
+        expected_rPr_xml = (
+            '<a:rPr xmlns:a="http://schemas.openxmlformats.org/drawingml/2006'
+            '/main" b="1"/>')
         # exercise --------------------
         self.font.bold = True
         # verify ----------------------
-        rPr_xml = etree.tostring(self.font._Font__rPr)
+        rPr_xml = oxml_tostring(self.font._Font__rPr)
         assert_that(rPr_xml, is_(equal_to(expected_rPr_xml)))
 
     def test_clear_bold(self):
         """Setting _Font.bold to False selects normal font weight"""
         # setup -----------------------
-        rPr_xml = '<a:rPr%s b="1"/>' % nsprefix_decls
-        rPr = etree.fromstring(rPr_xml)
+        rPr_xml = (
+            '<a:rPr xmlns:a="http://schemas.openxmlformats.org/drawingml/2006'
+            '/main" b="1"/>')
+        rPr = oxml_fromstring(rPr_xml)
         font = _Font(rPr)
-        expected_rPr_xml = '<a:rPr%s/>'% (nsprefix_decls)
+        expected_rPr_xml = (
+            '<a:rPr xmlns:a="http://schemas.openxmlformats.org/drawingml/2006'
+            '/main"/>')
         # exercise --------------------
         font.bold = False
         # verify ----------------------
-        rPr_xml = etree.tostring(font._Font__rPr)
+        rPr_xml = oxml_tostring(font._Font__rPr)
         assert_that(rPr_xml, is_(equal_to(expected_rPr_xml)))
 
     def test_set_font_size(self):
         """Assignment to _Font.size changes font size"""
         # setup -----------------------
         newfontsize = 2400
-        expected_rPr_xml = '<a:rPr%s sz="%d"/>'% (nsprefix_decls, newfontsize)
+        expected_xml = (
+            '<a:rPr xmlns:a="http://schemas.openxmlformats.org/drawingml/2006'
+            '/main" sz="%d"/>') % newfontsize
         # exercise --------------------
         self.font.size = newfontsize
         # verify ----------------------
-        rPr_xml = etree.tostring(self.font._Font__rPr)
-        assert_that(rPr_xml, is_(equal_to(expected_rPr_xml)))
+        actual_xml = oxml_tostring(self.font._Font__rPr)
+        assert_that(actual_xml, is_(equal_to(expected_xml)))
 
 
 class TestImage(TestCase):
@@ -578,7 +580,7 @@ class TestImage(TestCase):
         """Image(path) constructor raises on bad path"""
         # verify ----------------------
         with self.assertRaises(IOError):
-            image = Image('foobar27.png')
+            Image('foobar27.png')
 
     def test___ext_from_image_stream_raises_on_incompatible_format(self):
         """Image.__ext_from_image_stream() raises on incompatible format"""
@@ -750,14 +752,14 @@ class TestParagraph(TestCase):
     """Test Paragraph"""
     def setUp(self):
         path = os.path.join(thisdir, 'test_files/slide1.xml')
-        self.sld = etree.parse(path).getroot()
+        self.sld = oxml_parse(path).getroot()
         xpath = './p:cSld/p:spTree/p:sp/p:txBody/a:p'
         self.pList = self.sld.xpath(xpath, namespaces=nsmap)
 
         self.test_text = 'test text'
-        self.p_xml = ('<a:p%s><a:r><a:t>%s</a:t></a:r></a:p>'
-            % (nsprefix_decls, self.test_text))
-        self.p = etree.fromstring(self.p_xml)
+        self.p_xml = ('<a:p%s><a:r><a:t>%s</a:t></a:r></a:p>' %
+                      (nsprefix_decls, self.test_text))
+        self.p = oxml_fromstring(self.p_xml)
         self.paragraph = Paragraph(self.p)
 
     def test_runs_size(self):
@@ -780,7 +782,7 @@ class TestParagraph(TestCase):
         p_elm = self.pList[0]
         paragraph = Paragraph(p_elm)
         # exercise --------------------
-        run = paragraph.add_run()
+        paragraph.add_run()
         # verify ----------------------
         expected = 1
         actual = len(paragraph.runs)
@@ -790,48 +792,52 @@ class TestParagraph(TestCase):
     def test_clear_removes_all_runs(self):
         """Paragraph.clear() removes all runs from paragraph"""
         # setup -----------------------
-        p_elm = self.pList[2]
-        etree.SubElement(p_elm, qtag('a:pPr'))
-        paragraph = Paragraph(p_elm)
-        expected = 2
-        actual = len(paragraph.runs)
-        msg = "expected pre-test run count %s, got %s" % (expected, actual)
-        self.assertEqual(expected, actual, msg)
+        p = self.pList[2]
+        _SubElement(p, 'a:pPr')
+        paragraph = Paragraph(p)
+        assert_that(len(paragraph.runs), is_(equal_to(2)))
         # exercise --------------------
         paragraph.clear()
         # verify ----------------------
-        expected = 0
-        actual = len(paragraph.runs)
-        msg = "expected run count %s, got %s" % (expected, actual)
-        self.assertEqual(expected, actual, msg)
+        assert_that(len(paragraph.runs), is_(equal_to(0)))
 
     def test_clear_preserves_paragraph_properties(self):
         """Paragraph.clear() preserves paragraph properties"""
         # setup -----------------------
-        p_xml = ('<a:p%s><a:pPr lvl="1"/><a:r><a:t>%s</a:t></a:r></a:p>'
-            % (nsprefix_decls, self.test_text))
-        p_elm = etree.fromstring(p_xml)
+        nsprefix_decls = (
+            ' xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/'
+            'main"')
+        p_xml = ('<a:p%s><a:pPr lvl="1"/><a:r><a:t>%s</a:t></a:r></a:p>' %
+                 (nsprefix_decls, self.test_text))
+        p_elm = oxml_fromstring(p_xml)
         paragraph = Paragraph(p_elm)
         expected_p_xml = '<a:p%s><a:pPr lvl="1"/></a:p>' % nsprefix_decls
         # exercise --------------------
         paragraph.clear()
         # verify ----------------------
-        p_xml = etree.tostring(paragraph._Paragraph__p)
+        p_xml = oxml_tostring(paragraph._Paragraph__p)
         assert_that(p_xml, is_(equal_to(expected_p_xml)))
 
     def test_set_font_size(self):
         """Assignment to Paragraph.font.size changes font size"""
         # setup -----------------------
         newfontsize = Pt(54.3)
-        expected_p_xml = (\
-            '<a:p%s><a:pPr><a:defRPr sz="%d"/></a:pPr><a:r><a:t>%s</a:t></a:r'
-            '></a:p>'
-            % (nsprefix_decls, newfontsize, self.test_text))
+        expected_xml = (
+            '<?xml version=\'1.0\' encoding=\'UTF-8\' standalone=\'yes\'?>\n<'
+            'a:p xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/ma'
+            'in">\n  <a:pPr>\n    <a:defRPr sz="5430"/>\n  </a:pPr>\n  <a:r>'
+            '\n    <a:t>test text</a:t>\n  </a:r>\n</a:p>\n')
         # exercise --------------------
         self.paragraph.font.size = newfontsize
         # verify ----------------------
-        p_xml = etree.tostring(self.paragraph._Paragraph__p)
-        assert_that(p_xml, is_(equal_to(expected_p_xml)))
+        p_xml = prettify_nsdecls(oxml_tostring(self.paragraph._Paragraph__p,
+            encoding='UTF-8', pretty_print=True, standalone=True))
+        p_xml_lines = p_xml.split('\n')
+        expected_xml_lines = expected_xml.split('\n')
+        for idx, line in enumerate(p_xml_lines):
+            # msg = '\n\n%s' % sld_xml
+            msg = "\n\nexpected:\n\n%s\n\nbut got:\n\n%s" % (expected_xml, p_xml)
+            self.assertEqual(line, expected_xml_lines[idx], msg)
 
     def test_text_setter_sets_single_run_text(self):
         """assignment to Paragraph.text creates single run containing value"""
@@ -872,10 +878,6 @@ class TestParagraph(TestCase):
 
 class TestPart(TestCase):
     """Test Part"""
-    def test_class_present(self):
-        """Part class present in presentation module"""
-        self.assertClassInModule(pptx.presentation, 'Part')
-
     def test_constructs_presentation_for_rt_officedocument(self):
         """Part() returns Presentation for RT_OFFICEDOCUMENT"""
         # setup -----------------------
@@ -984,11 +986,11 @@ class TestPresentation(TestCase):
         prs._relationships = relationships
         prs.partname = '/ppt/presentation.xml'
         path = os.path.join(thisdir, 'test_files/presentation.xml')
-        prs._element = etree.parse(path).getroot()
+        prs._element = oxml_parse(path).getroot()
         # exercise --------------------
         blob = prs._blob
         # verify ----------------------
-        presentation = etree.fromstring(blob)
+        presentation = oxml_fromstring(blob)
         sldIds = presentation.xpath('./p:sldIdLst/p:sldId', namespaces=nsmap)
         expected = ['rId3', 'rId4', 'rId5']
         actual = [sldId.get(qtag('r:id')) for sldId in sldIds]
@@ -1255,20 +1257,28 @@ class TestRun(TestCase):
         self.test_text = 'test text'
         self.r_xml = ('<a:r%s><a:t>%s</a:t></a:r>'
             % (nsprefix_decls, self.test_text))
-        self.r = etree.fromstring(self.r_xml)
+        self.r = oxml_fromstring(self.r_xml)
         self.run = Run(self.r)
 
     def test_set_font_size(self):
         """Assignment to Run.font.size changes font size"""
         # setup -----------------------
         newfontsize = 2400
-        expected_r_xml = ('<a:r%s><a:rPr sz="%d"/><a:t>%s</a:t></a:r>'
-            % (nsprefix_decls, newfontsize, self.test_text))
+        expected_xml = ('<?xml version=\'1.0\' encoding=\'UTF-8\' standalone='
+            '\'yes\'?>\n<a:r xmlns:a="http://schemas.openxmlformats.org/drawi'
+            'ngml/2006/main">\n  <a:rPr sz="2400"/>\n  <a:t>test text</a:t>\n'
+            '</a:r>\n')
         # exercise --------------------
         self.run.font.size = newfontsize
         # verify ----------------------
-        r_xml = etree.tostring(self.run._Run__r)
-        assert_that(r_xml, is_(equal_to(expected_r_xml)))
+        r_xml = prettify_nsdecls(oxml_tostring(self.run._Run__r,
+            encoding='UTF-8', pretty_print=True, standalone=True))
+        r_xml_lines = r_xml.split('\n')
+        expected_xml_lines = expected_xml.split('\n')
+        for idx, line in enumerate(r_xml_lines):
+            msg = ("\n\nexpected:\n\n%s\n\nbut got:\n\n%s"
+                % (expected_xml, r_xml))
+            self.assertEqual(line, expected_xml_lines[idx], msg)
 
     def test_text_value(self):
         """Run.text value is correct"""
@@ -1309,16 +1319,12 @@ class TestShape(TestCase):
         sp = sldLayout.xpath('p:cSld/p:spTree/p:sp', namespaces=nsmap)[0]
         return Shape(sp)
 
-    def test_class_present(self):
-        """Shape class present in presentation module"""
-        self.assertClassInModule(pptx.presentation, 'Shape')
-
 
 class TestShapeCollection(TestCase):
     """Test ShapeCollection"""
     def setUp(self):
         path = absjoin(test_file_dir, 'slide1.xml')
-        sld = etree.parse(path).getroot()
+        sld = oxml_parse(path).getroot()
         spTree = sld.xpath('./p:cSld/p:spTree', namespaces=nsmap)[0]
         self.shapes = ShapeCollection(spTree)
 
@@ -1408,16 +1414,15 @@ class TestShapeCollection(TestCase):
         shapes = ShapeCollection(_empty_spTree())
         # exercise --------------------
         shape = shapes.add_textbox(left, top, width, height)
-        # setup -----------------------
-        xml = etree.tostring(shape._element, encoding='UTF-8',
-                             pretty_print=True, standalone=True)
+        # verify ----------------------
+        xml = oxml_tostring(shape._element, encoding='UTF-8',
+                            pretty_print=True, standalone=True)
         xml = prettify_nsdecls(xml)
         xml_lines = xml.split('\n')
         txbox_xml_lines = _txbox_xml().split('\n')
-        # verify ----------------------
-        # self.assertTrue(False, xml)
         for idx, line in enumerate(xml_lines):
-            assert_that(line, is_(equal_to(txbox_xml_lines[idx])))
+            msg = "expected:\n%s\n\nbut got:\n\n%s" % (xml, _txbox_xml())
+            self.assertEqual(line, txbox_xml_lines[idx], msg)
 
     def test_title_value(self):
         """ShapeCollection.title value is ref to correct shape"""
@@ -1486,25 +1491,69 @@ class TestShapeCollection(TestCase):
         layout_shapes = _sldLayout1_shapes()
         layout_ph_shapes = [sp for sp in layout_shapes if sp.is_placeholder]
         shapes = ShapeCollection(_empty_spTree())
-        expected_values =\
-            ( [2, 'Title 1',                    PH_TYPE_CTRTITLE,  0]
-            , [3, 'Date Placeholder 2',         PH_TYPE_DT,       10]
-            , [4, 'Vertical Subtitle 3',        PH_TYPE_SUBTITLE,  1]
-            , [5, 'Table Placeholder 4',        PH_TYPE_TBL,      14]
-            , [6, 'Slide Number Placeholder 5', PH_TYPE_SLDNUM,   12]
-            , [7, 'Footer Placeholder 6',       PH_TYPE_FTR,      11]
-            )
+        expected_values = (
+            [2, 'Title 1',                    PH_TYPE_CTRTITLE,  0],
+            [3, 'Date Placeholder 2',         PH_TYPE_DT,       10],
+            [4, 'Vertical Subtitle 3',        PH_TYPE_SUBTITLE,  1],
+            [5, 'Table Placeholder 4',        PH_TYPE_TBL,      14],
+            [6, 'Slide Number Placeholder 5', PH_TYPE_SLDNUM,   12],
+            [7, 'Footer Placeholder 6',       PH_TYPE_FTR,      11])
         # exercise --------------------
         for idx, layout_ph_sp in enumerate(layout_ph_shapes):
             layout_ph = Placeholder(layout_ph_sp)
             sp = shapes._ShapeCollection__clone_layout_placeholder(layout_ph)
-            # verify ----------------------
+            # verify ------------------
             ph = Placeholder(sp)
             expected = expected_values[idx]
             actual = [ph.id, ph.name, ph.type, ph.idx]
             msg = "expected placeholder values %s, got %s" % (expected, actual)
             self.assertEqual(expected, actual, msg)
 
+    def test___clone_layout_placeholder_xml(self):
+        """ShapeCollection.__clone_layout_placeholder() produces correct XML"""
+        # setup -----------------------
+        layout_shapes = _sldLayout1_shapes()
+        layout_ph_shapes = [sp for sp in layout_shapes if sp.is_placeholder]
+        shapes = ShapeCollection(_empty_spTree())
+        xml_template = (
+            '<?xml version=\'1.0\' encoding=\'UTF-8\' standalone='
+            '\'yes\'?>\n<p:sp xmlns:p="http://schemas.openxmlformats.org/pres'
+            'entationml/2006/main"\n      xmlns:a="http://schemas.openxmlform'
+            'ats.org/drawingml/2006/main">\n  <p:nvSpPr>\n    <p:cNvPr id="%d'
+            '" name="%s"/>\n    <p:cNvSpPr>\n      <a:spLocks noGrp="1"/>\n  '
+            '  </p:cNvSpPr>\n    <p:nvPr>\n      <p:ph type="%s"%s/>\n    </p'
+            ':nvPr>\n  </p:nvSpPr>\n  <p:spPr/>\n%s</p:sp>')
+        txBody_snippet = (
+            '  <p:txBody>\n    <a:bodyPr/>\n    <a:lstStyle/>\n    <a:p/>\n  '
+            '</p:txBody>\n')
+        expected_values = [
+            (2, 'Title 1', PH_TYPE_CTRTITLE, '', txBody_snippet),
+            (3, 'Date Placeholder 2', PH_TYPE_DT, ' sz="half" idx="10"', ''),
+            (4, 'Vertical Subtitle 3', PH_TYPE_SUBTITLE,
+                ' orient="vert" idx="1"', txBody_snippet),
+            (5, 'Table Placeholder 4', PH_TYPE_TBL,
+                ' sz="quarter" idx="14"', ''),
+            (6, 'Slide Number Placeholder 5', PH_TYPE_SLDNUM,
+                ' sz="quarter" idx="12"', ''),
+            (7, 'Footer Placeholder 6', PH_TYPE_FTR,
+                ' sz="quarter" idx="11"', '')]
+                    # verify ----------------------
+        for idx, layout_ph_sp in enumerate(layout_ph_shapes):
+            # log.debug("layout_ph_sp.name '%s'" % layout_ph_sp.name)
+            layout_ph = Placeholder(layout_ph_sp)
+            sp = shapes._ShapeCollection__clone_layout_placeholder(layout_ph)
+            ph = Placeholder(sp)
+            sp_xml = prettify_nsdecls(
+                oxml_tostring(ph._element, encoding='UTF-8',
+                              pretty_print=True, standalone=True))
+            sp_xml_lines = sp_xml.split('\n')
+            expected_xml = xml_template % expected_values[idx]
+            expected_xml_lines = expected_xml.split('\n')
+            for idx, line in enumerate(sp_xml_lines):
+                msg = '\n\n%s' % sp_xml
+                self.assertEqual(line, expected_xml_lines[idx], msg)
+                # assert_that(line, is_(equal_to(expected_xml_lines[idx])))
+
     def test___next_ph_name_return_value(self):
         """
         ShapeCollection.__next_ph_name() returns correct value
@@ -1514,12 +1563,11 @@ class TestShapeCollection(TestCase):
         * prefix 'Vertical' if orient="vert"
 
         """
-        cases =\
-            ( (PH_TYPE_OBJ,   3, PH_ORIENT_HORZ, 'Content Placeholder 2')
-            , (PH_TYPE_TBL,   4, PH_ORIENT_HORZ, 'Table Placeholder 4')
-            , (PH_TYPE_TBL,   7, PH_ORIENT_VERT, 'Vertical Table Placeholder 6')
-            , (PH_TYPE_TITLE, 2, PH_ORIENT_HORZ, 'Title 2')
-            )
+        cases = (
+            (PH_TYPE_OBJ,   3, PH_ORIENT_HORZ, 'Content Placeholder 2'),
+            (PH_TYPE_TBL,   4, PH_ORIENT_HORZ, 'Table Placeholder 4'),
+            (PH_TYPE_TBL,   7, PH_ORIENT_VERT, 'Vertical Table Placeholder 6'),
+            (PH_TYPE_TITLE, 2, PH_ORIENT_HORZ, 'Title 2'))
         # setup -----------------------
         shapes = _sldLayout1_shapes()
         for ph_type, id, orient, expected_name in cases:
@@ -1549,29 +1597,39 @@ class TestShapeCollection(TestCase):
         # setup -----------------------
         test_image = PILImage.open(test_image_path)
         pic_size = tuple(Px(x) for x in test_image.size)
-        xml = ('<p:pic xmlns:a="http://schemas.openxmlformats.org/drawingml/2'
-            '006/main" xmlns:p="http://schemas.openxmlformats.org/presentatio'
-            'nml/2006/main" xmlns:r="http://schemas.openxmlformats.org/office'
-            'Document/2006/relationships"><p:nvPicPr><p:cNvPr id="4" name="Pi'
-            'cture 3" descr="python-icon.jpeg"/><p:cNvPicPr/><p:nvPr/></p:nvP'
-            'icPr><p:blipFill><a:blip r:embed="rId9"/><a:stretch><a:fillRect/'
-            '></a:stretch></p:blipFill><p:spPr><a:xfrm><a:off x="0" y="0"/><a'
-            ':ext cx="%s" cy="%s"/></a:xfrm><a:prstGeom prst="rect"'
-            '><a:avLst/></a:prstGeom></p:spPr></p:pic>' % pic_size)
+        xml = (
+            '<?xml version=\'1.0\' encoding=\'UTF-8\' standalone=\'yes\'?>'
+            '\n<p:pic xmlns:a="http://schemas.openxmlformats.org/drawingml/20'
+            '06/main"\n       xmlns:p="http://schemas.openxmlformats.org/pres'
+            'entationml/2006/main"\n       xmlns:r="http://schemas.openxmlfor'
+            'mats.org/officeDocument/2006/relationships">\n  <p:nvPicPr>\n   '
+            ' <p:cNvPr id="4" name="Picture 3" descr="python-icon.jpeg"/>\n  '
+            '  <p:cNvPicPr/>\n    <p:nvPr/>\n  </p:nvPicPr>\n  <p:blipFill>\n'
+            '    <a:blip r:embed="rId9"/>\n    <a:stretch>\n      <a:fillRect'
+            '/>\n    </a:stretch>\n  </p:blipFill>\n  <p:spPr>\n    <a:xfrm>'
+            '\n      <a:off x="0" y="0"/>\n      <a:ext cx="%s" cy="%s"/>\n  '
+            '  </a:xfrm>\n    <a:prstGeom prst="rect">\n      <a:avLst/>\n   '
+            ' </a:prstGeom>\n  </p:spPr>\n</p:pic>' % pic_size)
         # exercise --------------------
         pic = self.shapes._ShapeCollection__pic('rId9', test_image_path, 0, 0)
         # verify ----------------------
-        expected = xml
-        actual = etree.tostring(pic)
-        msg = "\nExpected: %s\n     Got: %s" % (expected, actual)
-        self.assertEqual(expected, actual, msg)
+        pic_xml = oxml_tostring(pic, encoding='UTF-8', pretty_print=True,
+                                standalone=True)
+        pic_xml = prettify_nsdecls(pic_xml)
+        pic_xml_lines = pic_xml.split('\n')
+        expected_xml_lines = xml.split('\n')
+        for idx, line in enumerate(pic_xml_lines):
+            msg = "\n\nexpected:\n\n%s\n\nbut got\n\n%s" % (xml, pic_xml)
+            self.assertEqual(line, expected_xml_lines[idx], msg)
+            # assert_that(line, is_(equal_to(expected_xml_lines[idx])))
 
     def test___pic_from_stream_generates_correct_xml(self):
         """ShapeCollection.__pic returns correct XML from stream image"""
         # setup -----------------------
         test_image = PILImage.open(test_image_path)
         pic_size = tuple(Px(x) for x in test_image.size)
-        xml = ('<p:pic xmlns:a="http://schemas.openxmlformats.org/drawingml/2'
+        xml = (
+            '<p:pic xmlns:a="http://schemas.openxmlformats.org/drawingml/2'
             '006/main" xmlns:p="http://schemas.openxmlformats.org/presentatio'
             'nml/2006/main" xmlns:r="http://schemas.openxmlformats.org/office'
             'Document/2006/relationships"><p:nvPicPr><p:cNvPr id="4" name="Pi'
@@ -1584,7 +1642,7 @@ class TestShapeCollection(TestCase):
         with open(test_image_path) as stream:
             pic = self.shapes._ShapeCollection__pic('rId9', stream, 0, 0)
         # verify ----------------------
-        assert_that(etree.tostring(pic), is_(equal_to(xml)))
+        assert_that(oxml_tostring(pic), is_(equal_to(xml)))
 
 
 class TestSlide(TestCase):
@@ -1592,10 +1650,6 @@ class TestSlide(TestCase):
     def setUp(self):
         self.sld = Slide()
 
-    def test_class_present(self):
-        """Slide class present in presentation module"""
-        self.assertClassInModule(pptx.presentation, 'Slide')
-
     def test_constructor_sets_correct_content_type(self):
         """Slide constructor sets correct content type"""
         # exercise --------------------
@@ -1635,9 +1689,10 @@ class TestSlide(TestCase):
         # verify ----------------------
         with open(path, 'r') as f:
             expected = f.read()
-        actual = etree.tostring(elm, encoding='UTF-8', pretty_print=True,
-                                                         standalone=True)
-        msg = "expected:\n%s\n, got\n%s" % (expected, actual)
+        actual = prettify_nsdecls(
+            oxml_tostring(elm, encoding='UTF-8', pretty_print=True,
+                          standalone=True))
+        msg = "\nexpected:\n\n'%s'\n\nbut got:\n\n'%s'" % (expected, actual)
         self.assertEqual(expected, actual, msg)
 
     def test_slidelayout_property_none_on_construction(self):
@@ -1669,6 +1724,25 @@ class TestSlide(TestCase):
         msg = "expected: %s, got %s" % (expected, actual)
         self.assertEqual(expected, actual, msg)
 
+    def test___minimal_element_xml(self):
+        """Slide.__minimal_element generates correct XML"""
+        # setup -----------------------
+        path = os.path.join(thisdir, 'test_files/minimal_slide.xml')
+        # exercise --------------------
+        sld = self.sld._Slide__minimal_element
+        # verify ----------------------
+        with open(path, 'r') as f:
+            expected_xml = f.read()
+        sld_xml = prettify_nsdecls(
+            oxml_tostring(sld, encoding='UTF-8', pretty_print=True,
+                          standalone=True))
+        sld_xml_lines = sld_xml.split('\n')
+        expected_xml_lines = expected_xml.split('\n')
+        for idx, line in enumerate(sld_xml_lines):
+            # msg = '\n\n%s' % sld_xml
+            msg = "expected:\n%s\n, got\n%s" % (expected_xml, sld_xml)
+            self.assertEqual(line, expected_xml_lines[idx], msg)
+
 
 class TestSlideCollection(TestCase):
     """Test SlideCollection"""
@@ -1777,10 +1851,6 @@ class TestSlideLayout(TestCase):
         slidelayout = SlideLayout()
         return slidelayout._load(pkg_slidelayout_part, loaded_part_dict)
 
-    def test_class_present(self):
-        """SlideLayout class present in presentation module"""
-        self.assertClassInModule(pptx.presentation, 'SlideLayout')
-
     def test__load_sets_slidemaster(self):
         """SlideLayout._load() sets slidemaster"""
         # setup -----------------------
@@ -1809,10 +1879,6 @@ class TestSlideMaster(TestCase):
     def setUp(self):
         self.sldmaster = SlideMaster()
 
-    def test_class_present(self):
-        """SlideMaster class present in presentation module"""
-        self.assertClassInModule(pptx.presentation, 'SlideMaster')
-
     def test_slidelayouts_property_empty_on_construction(self):
         """SlideMaster.slidelayouts property empty on construction"""
         # verify ----------------------
@@ -1833,8 +1899,7 @@ class TestTextFrame(TestCase):
     """Test TextFrame"""
     def setUp(self):
         path = os.path.join(thisdir, 'test_files/slide1.xml')
-        parser = etree.XMLParser(remove_blank_text=True)
-        self.sld = etree.parse(path, parser).getroot()
+        self.sld = oxml_parse(path).getroot()
         xpath = './p:cSld/p:spTree/p:sp/p:txBody'
         self.txBodyList = self.sld.xpath(xpath, namespaces=nsmap)
 
@@ -1852,7 +1917,7 @@ class TestTextFrame(TestCase):
         msg = "expected paragraph count %s, got %s" % (expected, actual)
         self.assertEqual(expected, actual, msg)
 
-    def test_add_paragraph_works(self):
+    def test_add_paragraph_xml(self):
         """TextFrame.add_paragraph does what it says"""
         # setup -----------------------
         txBody_xml = (
@@ -1864,16 +1929,15 @@ class TestTextFrame(TestCase):
         expected_xml = (
             '<p:txBody xmlns:p="http://schemas.openxmlformats.org/presentatio'
             'nml/2006/main" xmlns:a="http://schemas.openxmlformats.org/drawin'
-            'gml/2006/main" xmlns:r="http://schemas.openxmlformats.org/office'
-            'Document/2006/relationships"><a:bodyPr/><a:p><a:r><a:t>Test text'
-            '</a:t></a:r></a:p><a:p/></p:txBody>')
-        txBody = etree.fromstring(txBody_xml)
+            'gml/2006/main"><a:bodyPr/><a:p><a:r><a:t>Test text</a:t></a:r></'
+            'a:p><a:p/></p:txBody>')
+        txBody = oxml_fromstring(txBody_xml)
         textframe = TextFrame(txBody)
         # exercise --------------------
-        p = textframe.add_paragraph()
+        textframe.add_paragraph()
         # verify ----------------------
         assert_that(len(textframe.paragraphs), is_(equal_to(2)))
-        textframe_xml = etree.tostring(textframe._TextFrame__txBody)
+        textframe_xml = oxml_tostring(textframe._TextFrame__txBody)
         expected = expected_xml
         actual = textframe_xml
         msg = "\nExpected: '%s'\n\n     Got: '%s'" % (expected, actual)
@@ -1902,28 +1966,29 @@ class TestTextFrame(TestCase):
     def test_vertical_anchor_works(self):
         """Assignment to TextFrame.vertical_anchor sets vert anchor"""
         # setup -----------------------
-        txBody_xml = (\
+        txBody_xml = (
             '<p:txBody xmlns:p="http://schemas.openxmlformats.org/presentatio'
             'nml/2006/main" xmlns:a="http://schemas.openxmlformats.org/drawin'
-            'gml/2006/main" xmlns:r="http://schemas.openxmlformats.org/office'
-            'Document/2006/relationships"><a:bodyPr/><a:p><a:r><a:t>Test text'
-            '</a:t></a:r></a:p></p:txBody>')
-        expected_xml = (\
+            'gml/2006/main"><a:bodyPr/><a:p><a:r><a:t>Test text</a:t></a:r></'
+            'a:p></p:txBody>')
+        expected_xml = (
             '<p:txBody xmlns:p="http://schemas.openxmlformats.org/presentatio'
             'nml/2006/main" xmlns:a="http://schemas.openxmlformats.org/drawin'
-            'gml/2006/main" xmlns:r="http://schemas.openxmlformats.org/office'
-            'Document/2006/relationships"><a:bodyPr anchor="ctr"/><a:p><a:r><'
-            'a:t>Test text</a:t></a:r></a:p></p:txBody>')
-        txBody = etree.fromstring(txBody_xml)
+            'gml/2006/main">\n  <a:bodyPr anchor="ctr"/>\n  <a:p>\n    <a:r>'
+            '\n      <a:t>Test text</a:t>\n    </a:r>\n  </a:p>\n</p:txBody>'
+            '\n')
+        txBody = oxml_fromstring(txBody_xml)
         textframe = TextFrame(txBody)
         # exercise --------------------
         textframe.vertical_anchor = MSO.ANCHOR_MIDDLE
         # verify ----------------------
-        textframe_xml = etree.tostring(textframe._TextFrame__txBody)
-        expected = expected_xml
-        actual = textframe_xml
-        msg = "\nExpected: '%s'\n     Got: '%s'" % (expected, actual)
-        if not expected == actual:
-            raise AssertionError(msg)
-
+        expected_xml_lines = expected_xml.split('\n')
+        actual_xml = oxml_tostring(textframe._TextFrame__txBody,
+                                   pretty_print=True)
+        actual_xml_lines = actual_xml.split('\n')
+        for idx, actual_line in enumerate(actual_xml_lines):
+            expected_line = expected_xml_lines[idx]
+            msg = ("\n\nexpected:\n\n'%s'\n\nbut got:\n\n'%s'"
+                   % (expected_xml, actual_xml))
+            assert_that(actual_line, is_(equal_to(expected_line)), msg)
 
